import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { is, fromJS } from 'immutable'
import { Spin, Empty, Row, Col, Pagination, Switch, notification } from 'antd'
import { DownOutlined, UpOutlined, PlusSquareOutlined, MinusSquareOutlined } from '@ant-design/icons'

import Api from '@/api'
import UtilsDM from '@/utils/utils-datamanage.js'
import MKEmitter from '@/utils/events.js'
import TimerTask from '@/utils/timer-task.js'
import asyncComponent from '@/utils/asyncComponent'
import './index.scss'

const CardItem = asyncComponent(() => import('../cardItem'))
const TableHeader = asyncComponent(() => import('../tableHeader'))
const MainAction = asyncComponent(() => import('@/tabviews/zshare/actionList'))
const NormalHeader = asyncComponent(() => import('@/tabviews/custom/components/share/normalheader'))

class DoubleDataCard extends Component {
  static propTpyes = {
    config: PropTypes.object,        // 组件配置信息
  }

  state = {
    BID: '',                   // 上级ID
    BData: '',                 // 上级行数据
    config: null,              // 图表配置信息
    search: null,              // 搜索条件
    pageIndex: 1,              // 页码
    pageSize: 10,
    orderBy: '',
    pageOptions: [],
    activeKey: '',             // 选中卡
    selectKeys: [],            // 多选时选中卡片
    selectedData: [],          // 选中数据，用于工具栏按钮
    loading: false,            // 数据加载状态
    card: null,                // 卡片设置
    data: [],                // 数据
    total: 0,
    precards: [],
    nextcards: [],
    selected: 'false',
    opens: [],
    wrapStyle: null,
    subcard: null,
    subconfig: null,
    pickup: false
  }

  loaded = false

  UNSAFE_componentWillMount () {
    const { config } = this.props

    let _config = fromJS(config).toJS()
    let BID = ''
    let BData = ''

    if (_config.setting.supModule) {
      BData = window.GLOB.CacheData.get(_config.setting.supModule)
    } else {
      BData = window.GLOB.CacheData.get(_config.$pageId)
    }
    if (BData) {
      BID = BData.$BID || ''
    }

    let _card = null
    let precards = []
    let nextcards = []

    if (_config.wrap.controlField) {
      if (_config.wrap.controlVal) {
        _config.wrap.controlVal = _config.wrap.controlVal.split(',')
      } else {
        _config.wrap.controlVal = ['']
      }
    }

    _config.$extend = false
    _config.$empty = true

    _config.subcards.forEach(item => {
      if (item.setting.click === 'button' && !item.setting.linkbtn) {
        item.elements.forEach(ele => {
          if (ele.eleType === 'button') {
            item.setting.linkbtn = ele.uuid
          }
        })
        if (!item.setting.linkbtn) {
          item.setting.click = ''
        }
      }

      if (item.style.clear === 'left') {
        item.wStyle = {clear: 'left'}
      } else if (item.style.clear === 'right') {
        item.wStyle = {float: 'right'}
      }

      delete item.style.clear

      if (item.$cardType !== 'extendCard') {
        _card = item
      } else if (!_card) {
        _config.$extend = true
        if (item.setting.width !== 24) {
          _config.$empty = false
        }
        precards.push(item)
      } else {
        _config.$extend = true
        _config.$empty = false
        nextcards.push(item)
      }
    })

    _config.subcards = null

    if (!_config.wrap.cardType) {
      _config.wrap.selStyle = 'none'
      _config.wrap.selected = 'false'
      _config.wrap.pickup = 'false'
    }
    
    let selected = 'false'
    if (_config.wrap.selected === 'always' || _config.wrap.selected === 'init' || _config.wrap.selected === 'sign') {
      selected = _config.wrap.selected
    } else {
      _config.wrap.selected = 'false'
    }

    _config.wrap.selStyle = _config.wrap.selStyle || 'check'
    _config.wrap.pagestyle = _config.wrap.pagestyle || 'page'

    _config.wrap.wrapClass =  `${_config.wrap.selStyle} ${_config.wrap.cardType || ''} ${config.wrap.empSign === 'hidden' ? 'mk-empty-hide' : ''}`

    if (_config.wrap.zHeight || _config.wrap.minWidth) {
      _config.wrap.zoomStyle = {
        maxHeight: 'none',
        '--mk-data-zoom-width': _config.wrap.minWidth ? _config.wrap.minWidth + 'px' : '100%'
      }

      if (_config.wrap.zHeight) {
        if (_config.wrap.zHeight <= 100) {
          if (_config.wrap.zHeight < 0) {
            _config.wrap.zoomStyle.maxHeight = `calc(100vh - ${-_config.wrap.zHeight}px)`
            _config.wrap.zoomStyle.minHeight = '100px'
          } else {
            _config.wrap.zoomStyle.maxHeight = _config.wrap.zHeight + 'vh'
          }
        } else {
          _config.wrap.zoomStyle.maxHeight = _config.wrap.zHeight + 'px'
        }
        _config.wrap.wrapClass += ' fix-height'
      }
    } else {
      _config.wrap.zoomStyle = null
    }

    if (_config.wrap.shifting === 'true') {
      _config.wrap.shifting = 'shifting'
    } else {
      _config.wrap.shifting = ''
    }
    
    let wrapStyle = null
    let subcard = fromJS(_card).toJS()
    let subconfig = fromJS(_config).toJS()

    subconfig.columns = subconfig.subColumns || []
    subconfig.setting.primaryKey = subconfig.setting.subKey

    subcard.style = subcard.backStyle
    subcard.elements = subcard.backElements
    subcard.setting = subcard.backSetting

    if (_card.setting.position === 'inner') {
      wrapStyle = {backgroundColor: '#ffffff'}
      Object.keys(_card.style).forEach(key => {
        if (!/^(margin|border|box|backgroundColor)/.test(key)) return
        wrapStyle[key] = _card.style[key]
        delete _card.style[key]
      })
    }

    if (_config.setting.subdata !== 'sub_data_string') {
      _config.setting.sub_field = subconfig.columns.map(col => col.field).join(',')
      _config.setting.all_field = _config.setting.arr_field + ',' + _config.setting.sub_field
    }

    let pageOptions = ['10', '25', '50', '100', '500', '1000']

    if (!_config.setting.laypage) {
      _config.wrap.pagestyle = 'none'
    }

    if (_config.wrap.pagestyle === 'page') {
      let size = (_config.setting.pageSize || 10) + ''
      if (!pageOptions.includes(size)) {
        pageOptions.push(size)
        pageOptions = pageOptions.sort((a, b) => a - b)
      }
      if (_config.wrap.maxPageSize) {
        pageOptions = pageOptions.filter(item => item <= _config.wrap.maxPageSize)
      }
    }

    if (_config.setting.supModule && !BID && _config.wrap.supKey !== 'false') {
      _config.setting.onload = 'false'
    }

    this.setState({
      pageSize: _config.setting.pageSize || 10,
      pageOptions,
      selected,
      precards,
      nextcards,
      BID: BID || '',
      BData: BData || '',
      config: _config,
      subcard: subcard,
      subconfig: subconfig,
      wrapStyle: wrapStyle,
      card: _card,
      search: _config.$searches
    })
  }

  componentDidMount () {
    const { config } = this.state

    MKEmitter.addListener('reloadData', this.reloadData)
    MKEmitter.addListener('refreshLineData', this.refreshLineData)
    MKEmitter.addListener('resetSelectLine', this.resetParentParam)
    MKEmitter.addListener('queryModuleParam', this.queryModuleParam)
    MKEmitter.addListener('refreshByButtonResult', this.refreshByButtonResult)
    
    if (config.setting.useMSearch) {
      MKEmitter.addListener('searchRefresh', this.searchRefresh)
    }

    if (config.timer) {
      this.timer = new TimerTask()
      this.timer.init(config.uuid, config.timer, config.timerRepeats, () => {
        this.setState({
          pageIndex: 1
        }, () => {
          this.loadData('', 'timer')
        })
      })
    }

    if (config.setting.onload === 'true') {
      this.initExec()
    }

    this.autoExec()
  }

  shouldComponentUpdate (nextProps, nextState) {
    return !is(fromJS(this.state), fromJS(nextState))
  }

  componentWillUnmount () {
    this.setState = () => {
      return
    }
    MKEmitter.removeListener('reloadData', this.reloadData)
    MKEmitter.removeListener('searchRefresh', this.searchRefresh)
    MKEmitter.removeListener('refreshLineData', this.refreshLineData)
    MKEmitter.removeListener('resetSelectLine', this.resetParentParam)
    MKEmitter.removeListener('queryModuleParam', this.queryModuleParam)
    MKEmitter.removeListener('refreshByButtonResult', this.refreshByButtonResult)

    this.timer && this.timer.stop()
  }

  initExec = () => {
    const { config, BID } = this.state

    if (config.$cache) {
      let getData = (res) => {
        return res.map((item, index) => {
          let _children = item[config.setting.subdata]
          
          delete item[config.setting.subdata]
          if (config.setting.subdata === 'sub_data_string') {
            _children = this.parseSubData(_children)
          } else if (item.sub_data_string) {
            _children = this.parseSubData(item.sub_data_string)
          }

          let children = []

          if (_children) {
            _children.forEach((cell, i) => {
              cell.key = i
              cell.$$uuid = cell[config.setting.subKey] || ''
              cell.$$parentId = item[config.setting.primaryKey] || ''
              cell.$$BID = item[config.setting.primaryKey] || ''
              cell.$$BData = {...item}
              cell.$Index = i + 1 + ''

              if (cell.$$uuid) {
                children.push(cell)
              }
            })
          }

          item.children = children

          item.key = index
          item.$$uuid = item[config.setting.primaryKey] || ''
          item.$Index = index + 1 + ''

          if (config.wrap.controlField) {
            if (config.wrap.controlVal.includes(item[config.wrap.controlField] + '')) {
              item.$disabled = true
            }
          }

          return item
        })
      }
      
      if (config.$time && !config.setting.laypage) {
        Api.getLCacheConfig(config.uuid, config.$time, BID).then(res => {
          if (!res.valid) {
            setTimeout(() => {
              this.loadData('', 'init')
            }, config.setting.delay || 0)
          }

          if (!res.data) return
  
          this.setState({data: getData(res.data)})
        })
      } else {
        Api.getLCacheConfig(config.uuid, 0, BID).then(res => {
          if (!res.data || this.loaded) return
  
          this.setState({data: getData(res.data)})
        })

        setTimeout(() => {
          this.loadData('', 'init')
        }, config.setting.delay || 0)
      }
    } else {
      setTimeout(() => {
        this.loadData()
      }, config.setting.delay || 0)
    }
  }

  parseSubData = (data) => {
    try {
      data = JSON.parse(data)
      if (typeof(data) !== 'object') {
        data = []
      } else if (!Array.isArray(data)) {
        data = [data]
      }
    } catch(e) {
      console.warn('Data parsing error.')
      data = []
    }
    return data
  }

  autoExec = (times) => {
    const { config } = this.state

    if (!config.wrap.autoExec) return

    let btn = document.getElementById('button' + config.wrap.autoExec)

    this.autoTimer && clearTimeout(this.autoTimer)

    if (btn) {
      MKEmitter.emit('triggerBtnId', config.wrap.autoExec, [])
    } else if (!times || times < 20) {
      times = times ? times + 1 : 1
      this.autoTimer = setTimeout(() => {
        this.autoExec(times)
      }, 1000)
    }
  }

  searchRefresh = (searchId) => {
    const { config } = this.state

    if (config.$searchId !== searchId) return
    
    this.reloadIndex()
  }

  /**
   * @description 按钮执行完成后页面刷新
   */
  refreshByButtonResult = (menuId, position, btn, id = '', lines) => {
    const { config, BID } = this.state

    if (config.uuid !== menuId) return

    let supModule = config.setting.supModule

    let _id = config.wrap.selected === 'always' ? id : ''
    if (position === 'line' || position === 'line_grid') {
      if (lines && lines.length === 1) {
        if (config.forbidLine) {
          this.loadData(lines[0].$$parentId || lines[0].$$uuid, 'repage')
        } else {
          this.loadLinedata(lines[0].$$parentId || lines[0].$$uuid, position)
        }
      } else {
        this.loadData(_id, 'repage')
      }
    } else if (['mainline', 'maingrid', 'popclose'].includes(position) && supModule) {
      MKEmitter.emit('reloadData', supModule, position === 'maingrid' ? '' : BID)
    } else if (btn.resetPageIndex !== 'false') {
      this.reloadIndex()
    } else {
      this.loadData(_id, 'repage')
    }
  }

  refreshLineData = (menuId, btn, uuid, count) => {
    const { config, data } = this.state

    if (config.uuid !== menuId) return
    
    let _data = fromJS(data).toJS().map(item => {
      if (item.$$uuid === uuid) {
        item[btn.field] = count
      }
      item.children.forEach(cell => {
        if (cell.$$uuid === uuid) {
          cell[btn.field] = count
        }
      })
      return item
    })

    this.setState({
      data: _data
    })
  }

  checkTopLine = (id, selected) => {
    const { config, data, card } = this.state
    let _opens = [...this.state.opens]

    let index = -1
    let keys = []
    let items = []

    if (data.length > 0) {
      if (id) {
        index = data.findIndex(item => item.$$uuid === id && !item.$disabled)
      }

      if (index !== -1) {
        keys = [index]
        items = [data[index]]
      } else if (selected === 'sign') {
        data.forEach((item, i) => {
          if (!item.$disabled && item.selected === 'true') {
            items.push(item)
            keys.push(i)
            index = i
          }
        })
      } else if (selected !== 'false') {
        if (!data[0].$disabled) {
          index = 0
          keys = [index]
          items = [data[index]]
        }
      }
    }

    if (index === -1) {
      this.setState({
        activeKey: '',
        selectKeys: [],
        selectedData: []
      })
  
      MKEmitter.emit('resetSelectLine', config.uuid, '', '')
      return
    }

    if (card.setting.display === 'collapse') {
      _opens = keys
    }

    let item = items[items.length - 1]

    this.setState({
      opens: _opens,
      activeKey: index,
      selectKeys: keys,
      selectedData: items
    })

    MKEmitter.emit('resetSelectLine', config.uuid, item.$$uuid, item)
  }

  checkAll = () => {
    const { config, data, selectedData } = this.state

    if (config.wrap.cardType !== 'checkbox') return
    if (data.length === 0) return
    
    if (selectedData.length === 0 || selectedData.length < data.length) {
      let index = ''
      let keys = []
      let items = []
      let last = ''

      data.forEach((item, i) => {
        if (item.$disabled) return

        items.push(item)
        keys.push(i)
        index = i
        last = item
      })

      this.setState({
        activeKey: index,
        selectKeys: keys,
        selectedData: items
      })
  
      MKEmitter.emit('resetSelectLine', config.uuid, last ? last.$$uuid : '', last)
    } else {
      this.setState({
        activeKey: '',
        selectKeys: [],
        selectedData: []
      })
  
      MKEmitter.emit('resetSelectLine', config.uuid, '', '')
    }
  }

  reloadData = (menuId, id) => {
    const { config } = this.state

    if (config.uuid !== menuId) return

    if (!id) {
      this.reloadIndex()
    } else if (config.forbidLine) {
      this.loadData(id, 'repage')
    } else {
      this.loadLinedata(id)
    }
  }

  resetParentParam = (MenuID, id, data) => {
    const { config } = this.state

    if (!config.setting.supModule || config.setting.supModule !== MenuID) return
    if (id !== this.state.BID || id !== '') {
      this.setState({ BID: id, BData: data, pageIndex: 1 }, () => {
        if (!config.checkBid) {
          this.loadData()
        }
      })
    }
  }

  /**
   * @description 导出Excel时，获取页面搜索排序等参数
   */
  queryModuleParam = (menuId, callback) => {
    const { config, search, orderBy } = this.state

    if (config.uuid !== menuId) return

    let searches = search ? fromJS(search).toJS() : []
    if (config.setting.useMSearch) { // 主表搜索条件
      let mainSearch = window.GLOB.SearchBox.get(config.$searchId) || []
      let keys = config.$s_keys || []
      mainSearch.forEach(item => {
        if (keys.includes(item.key.toLowerCase())) return

        searches.push(item)
      })
    }

    callback({
      orderBy: orderBy || config.setting.order || '',
      search: searches
    })
  }

  async loadData (id, type) {
    const { config, pageIndex, pageSize, search, BID, BData, selected, card, orderBy } = this.state

    if (config.setting.supModule && !BID && config.wrap.supKey !== 'false') { // BID 不存在时，不做查询
      this.loaded = true
      this.requestId = ''

      this.setState({
        activeKey: '',
        selectKeys: [],
        selectedData: [],
        pageIndex: 1,
        data: [],
        opens: [],
        total: 0,
        loading: false
      })

      MKEmitter.emit('resetSelectLine', config.uuid, '', '')
      return
    }

    let searches = fromJS(search).toJS()
    if (config.setting.useMSearch) { // 主表搜索条件
      let mainSearch = window.GLOB.SearchBox.get(config.$searchId) || []
      let keys = config.$s_keys || []
      mainSearch.forEach(item => {
        if (keys.includes(item.key.toLowerCase())) return

        searches.push(item)
      })
    }

    if (config.$s_req && searches.filter(item => item.required && item.value === '').length > 0) {
      return
    }

    if (type !== 'timer') {
      this.setState({
        loading: true
      })
    }

    let _orderBy = orderBy || config.setting.order || ''
    let param = UtilsDM.getQueryDataParams(config.setting, searches, _orderBy, pageIndex, pageSize, BID)

    this.requestId = config.uuid + new Date().getTime()

    let result = await Api.genericInterface(param, '', '', this.requestId)
    if (result.status) {
      if (result.$requestId && this.requestId !== result.$requestId) return

      if (type === 'repage' && result.data.length === 0 && pageIndex > 1) {
        this.reloadIndex()
        return
      }

      let start = 1
      if (config.setting.laypage) {
        start = pageSize * (pageIndex - 1) + 1
      }

      this.loaded = true
      if (config.$cache && type === 'init') {
        Api.writeCacheConfig(config.uuid, result.data || [], BID)
      }

      if (selected !== 'false' || id) {
        setTimeout(() => {
          this.checkTopLine(id, selected)
        }, 10)
      } else {
        MKEmitter.emit('resetSelectLine', config.uuid, '', '')
      }

      let data = []
      let opens = []

      if (type === 'plus') {
        let _data = (this.state.data || []).concat(result.data || [])
        data = _data.map((item, index) => {
          let _children = item[config.setting.subdata]
          
          delete item[config.setting.subdata]
          if (config.setting.subdata === 'sub_data_string') {
            _children = this.parseSubData(_children)
          } else if (item.sub_data_string) {
            _children = this.parseSubData(item.sub_data_string)
          }

          let children = []
          if (_children) {
            _children.forEach((cell, i) => {
              cell.key = i
              cell.$$uuid = cell[config.setting.subKey] || ''
              cell.$$parentId = item[config.setting.primaryKey] || ''
              cell.$$BID = item[config.setting.primaryKey] || ''
              cell.$$BData = {...item}
              cell.$Index = i + 1 + ''

              if (cell.$$uuid) {
                children.push(cell)
              }
            })
          }
          
          item.children = children

          item.key = index
          item.$$uuid = item[config.setting.primaryKey] || ''
          item.$$BID = BID || ''
          item.$$BData = BData || ''
          item.$Index = index + 1 + ''

          if (config.wrap.controlField) {
            if (config.wrap.controlVal.includes(item[config.wrap.controlField] + '')) {
              item.$disabled = true
            }
          }
          
          return item
        })
      } else {
        data = result.data.map((item, index) => {
          let _children = item[config.setting.subdata]
          
          delete item[config.setting.subdata]
          if (config.setting.subdata === 'sub_data_string') {
            _children = this.parseSubData(_children)
          } else if (item.sub_data_string) {
            _children = this.parseSubData(item.sub_data_string)
          }

          let children = []

          if (_children) {
            _children.forEach((cell, i) => {
              cell.key = i
              cell.$$uuid = cell[config.setting.subKey] || ''
              cell.$$parentId = item[config.setting.primaryKey] || ''
              cell.$$BID = item[config.setting.primaryKey] || ''
              cell.$$BData = {...item}
              cell.$Index = i + 1 + ''

              if (cell.$$uuid) {
                children.push(cell)
              }
            })
          }

          item.children = children

          item.key = index
          item.$$uuid = item[config.setting.primaryKey] || ''
          item.$$BID = BID || ''
          item.$$BData = BData || ''
          item.$Index = index + start + ''

          if (config.wrap.controlField) {
            if (config.wrap.controlVal.includes(item[config.wrap.controlField] + '')) {
              item.$disabled = true
            }
          }
          return item
        })
      }

      if (card.setting.display === 'unfold') {
        opens = data.map(item => item.key)
      } else {
        opens = []
      }

      let total = result.total || 0
      if (config.setting.custompage && data.length) {
        total = data[data.length - 1].mk_total || 0
      }

      this.setState({
        activeKey: '',
        selectKeys: [],
        opens: opens,
        selectedData: [],
        data: data,
        total: total,
        pickup: false,
        loading: false
      })

      if (selected === 'init') {
        this.setState({selected: 'false'})
      }

      if (config.timer && config.clearField && result.data && result.data[0]) {
        let vals = (config.clearValue || '').split(',')
        if (vals.includes(result.data[0][config.clearField])) {
          this.timer && this.timer.stop()
        }
      }
      
      UtilsDM.querySuccess(result)
    } else {
      this.setState({
        loading: false
      })
      this.timer && this.timer.stop()

      UtilsDM.queryFail(result)
    }
  }

  /**
   * @description 获取单行数据
   */ 
  async loadLinedata (id, position) {
    const { config, pageIndex, pageSize, search, BID, BData, orderBy } = this.state

    let searches = fromJS(search).toJS()
    if (config.setting.useMSearch) { // 主表搜索条件
      let mainSearch = window.GLOB.SearchBox.get(config.$searchId) || []
      let keys = config.$s_keys || []
      mainSearch.forEach(item => {
        if (keys.includes(item.key.toLowerCase())) return

        searches.push(item)
      })
    }

    this.setState({
      loading: true
    })

    let _orderBy = orderBy || config.setting.order || ''
    let param = UtilsDM.getQueryDataParams(config.setting, searches, _orderBy, pageIndex, pageSize, BID, id)

    let result = await Api.genericInterface(param)
    if (result.status) {
      if (position === 'line_grid' && (!result.data || !result.data[0])) {
        this.reloadIndex()
        return
      }

      let data = fromJS(this.state.data).toJS()
      let selectedData = fromJS(this.state.selectedData).toJS()
      let selectKeys = fromJS(this.state.selectKeys).toJS()
      let activeKey = this.state.activeKey

      if (result.data && result.data[0]) {
        let _data = result.data[0]
        _data.$$uuid = _data[config.setting.primaryKey] || ''
        _data.$$BID = BID || ''
        _data.$$BData = BData || ''

        data = data.map(item => {
          if (item.$$uuid === _data.$$uuid) {
            let _children = _data[config.setting.subdata]

            delete _data[config.setting.subdata]

            if (config.setting.subdata === 'sub_data_string') {
              _children = this.parseSubData(_children)
            } else if (_data.sub_data_string) {
              _children = this.parseSubData(_data.sub_data_string)
            }

            let children = []

            if (_children) {
              _children.forEach((cell, i) => {
                cell.key = i
                cell.$$uuid = cell[config.setting.subKey] || ''
                cell.$$parentId = _data[config.setting.primaryKey] || ''
                cell.$$BID = _data[config.setting.primaryKey] || ''
                cell.$$BData = {..._data}
                cell.$Index = i + 1 + ''

                if (cell.$$uuid) {
                  children.push(cell)
                }
              })
            }

            _data.children = children
            _data.key = item.key
            _data.$Index = item.$Index
            return _data
          } else {
            return item
          }
        })

        if (!_data.$Index) {
          this.setState({
            loading: false
          })
          return
        }

        selectedData = selectedData.map(item => {
          if (item.$$uuid === _data.$$uuid) {
            return _data
          }
          return item
        })

        this.setState({
          data: data,
          selectedData: selectedData,
          loading: false
        })

        if (_data.key === activeKey) {
          MKEmitter.emit('resetSelectLine', config.uuid, _data.$$uuid || '', _data)
        }
      } else {
        let index = data.findIndex(item => item.$$uuid === id)

        if (index === -1) {
          this.setState({
            loading: false
          })
          return
        }

        data = data.filter(item => item.$$uuid !== id)
        data = data.map((item, i) => {
          item.key = i
          return item
        })
        
        let trans = activeKey === index
        let _item = null
        
        if (config.wrap.cardType) {
          selectKeys = selectKeys.filter(key => key !== index)
          selectKeys = selectKeys.map(key => key > index ? key - 1 : key)

          selectedData = selectKeys.map(key => data[key]).filter(Boolean)

          activeKey = selectKeys.length ? selectKeys[selectKeys.length - 1] : ''

          if (trans && selectedData.length) {
            _item = selectedData[selectedData.length - 1]
          }
        }

        this.setState({
          data: data,
          activeKey: activeKey,
          selectKeys: selectKeys,
          selectedData: selectedData,
          loading: false
        })

        if (trans) {
          MKEmitter.emit('resetSelectLine', config.uuid, (_item ? _item.$$uuid : ''), _item)
        }
      }
    } else {
      this.setState({
        loading: false
      })
      
      UtilsDM.queryFail(result)
    }
  }

  reloadIndex = () => {
    this.setState({
      pageIndex: 1
    }, () => {
      this.loadData()
    })
  }

  loadMore = () => {
    const { total, pageIndex, pageSize, loading } = this.state

    if (loading || pageSize * pageIndex >= total) {
      return
    }

    this.setState({
      pageIndex: pageIndex + 1
    }, () => {
      this.loadData('', 'plus')
    })
  }

  changePageIndex = (page) => {
    this.setState({
      pageIndex: page
    }, () => {
      this.loadData()
    })
  }

  refreshSearch = (list) => {
    const { config, BID } = this.state

    if (config.setting.supModule && !BID && config.wrap.supKey !== 'false') {
      notification.warning({
        top: 92,
        message: window.GLOB.dict['sup_key_req'] || '需要上级主键值！',
        duration: 5
      })
      this.setState({
        search: list
      })
      return
    }

    this.setState({
      search: list,
      pageIndex: 1
    }, () => {
      this.loadData()
    })
  }

  pageSizeChange = (current, size) => {
    this.setState({
      pageIndex: current,
      pageSize: size
    }, () => {
      this.loadData()
    })
  }
  
  changeCard = (index, item, subClass) => {
    const { config, selectKeys, selectedData, activeKey, opens, card } = this.state

    if (card.setting.click === 'unfold' && card.setting.clickType !== 'multi') {
      if (subClass === 'mk-unfold') {
        this.setState({opens: opens.filter(item => item !== index)})
      } else {
        this.setState({opens: [...opens, index]})
      }
    }

    if (!config.wrap.cardType) return
    if (item.$disabled) return
    
    let _selectKeys = []
    let _selectedData = []
    let _activeKey = ''
    let _item = item
    if (config.wrap.cardType === 'checkbox') {
      if (activeKey === index) {
        _selectKeys = selectKeys.filter(key => key !== index)
        _selectedData = selectedData.filter(cell => cell.key !== index)
        _activeKey = _selectKeys.slice(-1)[0]
        _item = _selectedData.slice(-1)[0] || ''
      } else if (selectKeys.indexOf(index) > -1) {
        _selectKeys = selectKeys.filter(key => key !== index)
        _selectedData = selectedData.filter(cell => cell.key !== index)
        _activeKey = activeKey
        _item = _selectedData.filter(cell => cell.key === activeKey)[0] || ''
      } else {
        _selectKeys = [...selectKeys, index]
        _selectedData = [...selectedData, item]
        _activeKey = index
      }
    } else {
      if (activeKey === index) return

      _selectedData = [item]
      _selectKeys = [index]
      _activeKey = index
    }

    this.setState({
      activeKey: _activeKey,
      selectKeys: _selectKeys,
      selectedData: _selectedData
    })

    MKEmitter.emit('resetSelectLine', config.uuid, (_item ? _item.$$uuid : ''), _item)
  }

  changeUnfold = (e, i, subClass) => {
    const { opens } = this.state

    e.stopPropagation()

    if (subClass === 'mk-disabled') return

    if (subClass === 'mk-unfold') {
      this.setState({opens: opens.filter(item => item !== i)})
    } else {
      this.setState({opens: [...opens, i]})
    }
  }

  changeSubCard = (item) => {
    const { subcard, card } = this.state

    if (subcard.setting.click || card.setting.position !== 'inner' || card.setting.click !== 'menu') return
    
    let menuId = card.setting.menu.slice(-1)[0]
    let menu = null

    if (window.GLOB.mkThdMenus.has(menuId)) {
      menu = {...window.GLOB.mkThdMenus.get(menuId)}
    } else if (card.setting.MenuID) {
      menu = {
        MenuID: card.setting.MenuID,
        MenuName: card.setting.MenuName,
        type: card.setting.tabType
      }
    }

    if (!menu) return

    menu.param = {$BID: item.$$uuid || ''}

    Object.keys(item).forEach(key => {
      if (/^\$/.test(key)) return
      if (key === 'children') return
      menu.param[key] = item[key]
    })

    MKEmitter.emit('modifyTabs', menu, true)
  }

  onDoubleClick = (i, subClass) => {
    const { opens, card } = this.state

    if (card.setting.click !== 'unfold' || card.setting.clickType !== 'multi') return

    if (subClass === 'mk-unfold') {
      this.setState({opens: opens.filter(item => item !== i)})
    } else {
      this.setState({opens: [...opens, i]})
    }
  }

  refreshByHeader = (sorter) => {
    this.setState({
      orderBy: sorter || ''
    }, () => {
      this.loadData()
    })
  }

  pickupChange = () => {
    this.setState({
      pickup: !this.state.pickup
    })
  }

  render() {
    const { config, precards, nextcards, loading, pageIndex, pageSize, total, card, activeKey, BID, BData, selectedData, selectKeys, subcard, subconfig, wrapStyle, opens, pickup } = this.state

    let data = this.state.data

    if (config.wrap.empty === 'hidden' && data.length === 0) return null

    let extendData = null

    if (config.$extend) {
      extendData = {$$BID: BID, $$BData: BData, $$selectedData: selectedData, $$type: 'extendCard'}
      
      if (data[0]) {
        if (selectedData[0]) {
          extendData = {...extendData, ...selectedData[0]}
        } else {
          extendData = {...extendData, ...data[0]}
        }
      }
    }

    let checkAll = ''
    let mainBox = ''
    if (config.wrap.selStyle && config.wrap.selStyle.indexOf('check') > -1) {
      if (selectedData.length > 0) {
        checkAll = selectedData.length < data.length ? ' half' : ' whole'
      }
      if (card.setting.controlIcon === 'left') {
        checkAll += ' mk-extend-icon'
      }
    } else if (card.setting.display !== 'default' && card.setting.controlIcon === 'left') {
      mainBox = 'flex-card '
    }

    if (pickup) {
      data = data.filter(item => selectKeys.includes(item.key))
    }

    return (
      <div className={'double-data-card-box ' + config.wrap.shifting} id={'anchor' + config.uuid} style={config.style}>
        {loading ?
          <div className="loading-mask">
            {data.length ? <div className="ant-spin-blur"></div> : null}
            <Spin />
          </div> : null
        }
        <NormalHeader config={config} BID={BID} refresh={this.refreshSearch} />
        {config.action && config.action.length > 0 ?
          <MainAction
            BID={BID}
            BData={BData}
            setting={config.setting}
            actions={config.action}
            columns={config.columns}
            selectedData={selectedData}
          /> : null
        }
        {config.wrap.pickup === 'true' && this.state.data.length > 0 ? <div className="pickup-wrap"><Switch title="收起" checkedChildren={window.GLOB.dict['open'] || '开'} unCheckedChildren={window.GLOB.dict['shut'] || '关'} checked={pickup} onChange={this.pickupChange} /></div> : null}
        <div className={config.wrap.zoomStyle ? 'data-zoom-box' : ''} style={config.wrap.zoomStyle}>
          <div className={`data-zoom ${config.wrap.wrapClass}`}>
            <Row className="card-row-list">
              {precards.map((item, index) => (
                <Col key={'pre' + index} className="extend-card" style={item.wStyle} span={item.setting.width || 6}>
                  {item.setting.cardRole === 'header' ? <TableHeader card={item} data={extendData} refresh={this.refreshByHeader}>
                    {item.setting.checkAll === 'show' ? <span onClick={this.checkAll} className={'circle-select' + checkAll}></span> : null}
                  </TableHeader> : <CardItem card={item} cards={config} data={extendData}>
                    {item.setting.checkAll === 'show' ? <span onClick={this.checkAll} className={'circle-select' + checkAll}></span> : null}
                  </CardItem>}
                </Col>
              ))}
              {data.map((item, index) => {
                let className = 'card-item-wrap mk-card ' + mainBox
                let subClass = 'mk-unfold'
                let unfold = true

                if (item.$disabled) {
                  className = 'card-item-wrap mk-disabled ' + mainBox
                } else if (activeKey === item.key) {
                  className += 'active'
                } else if (selectKeys.indexOf(item.key) > -1) {
                  className += 'selected'
                }

                if (card.setting.display !== 'default') {
                  if (item.children.length === 0) {
                    subClass = 'mk-disabled'
                    unfold = false
                  } else {
                    subClass = opens.indexOf(item.key) > -1 ? 'mk-unfold' : 'mk-collapse'
                    unfold = opens.indexOf(item.key) > -1
                  }

                  if (!unfold && item.children.length > 5) {
                    return (
                      <Col key={index} span={card.setting.width} style={card.wStyle}>
                        <div className={className} style={wrapStyle}>
                          <CardItem card={card} cards={config} data={item} onDoubleClick={() => this.onDoubleClick(item.key, subClass)} onClick={() => {this.changeCard(item.key, item, subClass)}}>
                            <span className="circle-select"></span>
                            {card.setting.controlIcon === 'left' ? <PlusSquareOutlined className={subClass} onClick={(e) => this.changeUnfold(e, item.key, subClass)}/> : null}
                            {card.setting.controlIcon === 'right' ? <UpOutlined className={subClass} onClick={(e) => this.changeUnfold(e, item.key, subClass)}/> : null}
                          </CardItem>
                        </div>
                      </Col>
                    )
                  }
                }

                return (
                  <Col key={index} span={card.setting.width} style={card.wStyle}>
                    <div className={className} style={wrapStyle}>
                      <CardItem card={card} cards={config} data={item} onDoubleClick={() => this.onDoubleClick(item.key, subClass)} onClick={() => {this.changeCard(item.key, item, subClass)}}>
                        <span className="circle-select"></span>
                        {card.setting.controlIcon === 'left' ? (!unfold ? <PlusSquareOutlined className={subClass} onClick={(e) => this.changeUnfold(e, item.key, subClass)}/> : <MinusSquareOutlined className={subClass} onClick={(e) => this.changeUnfold(e, item.key, subClass)}/>) : null}
                        {card.setting.controlIcon === 'right' ? <UpOutlined className={subClass} onClick={(e) => this.changeUnfold(e, item.key, subClass)}/> : null}
                      </CardItem>
                      <div className={'sub-card-wrap ' + subClass + (config.wrap.parity === 'true' ? ' mk-parity-bg' : '')} onClick={() => this.changeSubCard(item)}>
                        {item.children.map((cell, index) => <Col key={'sub' + index} span={subcard.setting.width || 24}>
                          <CardItem card={subcard} cards={subconfig} data={cell} />
                        </Col>)}
                      </div>
                    </div>
                  </Col>
                )
              })}
              {nextcards.map((item, index) => (
                <Col key={'next' + index} className="extend-card" style={item.wStyle} span={item.setting.width || 6}>
                  {item.setting.cardRole === 'header' ? <TableHeader card={item} data={extendData} refresh={this.refreshByHeader}>
                    {item.setting.checkAll === 'show' ? <span onClick={this.checkAll} className={'circle-select' + checkAll}></span> : null}
                  </TableHeader> : <CardItem card={item} cards={config} data={extendData}>
                    {item.setting.checkAll === 'show' ? <span onClick={this.checkAll} className={'circle-select' + checkAll}></span> : null}
                  </CardItem>}
                </Col>
              ))}
            </Row>
          </div>
          {config.$empty && data.length === 0 ? <Empty description={false}/> : null}
        </div>
        {config.wrap.pagestyle === 'page' ? <Pagination size="small" total={total} showTotal={(t, range) => total > 0 ? `${range[0]}-${range[1]} ${window.GLOB.dict['of'] || '共'} ${total} ${window.GLOB.dict['items'] || '条'}` : `${total} ${window.GLOB.dict['items'] || '条'}`} pageSize={pageSize} showSizeChanger={true} pageSizeOptions={this.state.pageOptions} onChange={this.changePageIndex} onShowSizeChange={this.pageSizeChange} current={pageIndex}/> : null}
        {config.wrap.pagestyle === 'more' && data.length > 0 ? <div className={'mk-more' + (pageSize * pageIndex >= total ? ' disabled' : '')} onClick={this.loadMore}>{window.GLOB.dict['view_more'] || '查看更多'}<DownOutlined/></div> : null}
      </div>
    )
  }
}

export default DoubleDataCard